//
//  Board.swift
//  GameBattleshipTerminal
//
//  Created by Piotr Fulmański on 2020.02.26.
//  Copyright © 2020 Piotr Fulmański. All rights reserved.
//

import Foundation

class Board {
    private let rows, cols: Int
    private var board: [[CellType]]

    enum CellType {
        case none, empty, hit, notAllowed, rescue, ship, shot
    }
    
    init (rows: Int, cols: Int) {
        self.rows = rows
        self.cols = cols
        
        board = Array(repeating: Array(repeating: .none, count: cols+2), count: rows+2)
        
        prepareBoard()
    }
    
    func prepareBoard() {
        for i in 0...rows+1 {
            board[i][0] = .notAllowed
            board[i][cols+1] = .notAllowed
        }
        
        for i in 0...cols+1 {
            board[0][i] = .notAllowed
            board[rows+1][i] = .notAllowed
        }
        
        for r in 1...rows {
            for c in 1...cols {
                board[r][c] = .empty
            }
        }
    }
    
    func mayPlaceShip(size: Int, anchor: (row: Int, col: Int), direction: Ship.Direction) -> Bool {
        var modifier: (forRow: Int, forCol: Int)!
        var r: Int!
        var c: Int!
        
        switch direction {
        case .up:
            modifier = (forRow: -1, forCol:  0)
        case .down:
            modifier = (forRow: +1, forCol:  0)
        case .left:
            modifier = (forRow:  0, forCol: -1)
        case .right:
            modifier = (forRow:  0, forCol: +1)
        }
        
        for i in 0...size-1 {
            r = anchor.row + i*modifier.forRow
            c = anchor.col + i*modifier.forCol
            
            guard r>0, r<rows+1 else {return false}
            guard c>0, c<cols+1 else {return false}
            
            if board[r][c] != .empty {
                return false
            }
        }
        
        return true
    }
        
    func printBoard() {
        let leadingPadding = EngineGameBattleshipUtils.determineNumberOfdigits(number: rows)
        var leadingPaddingString = ""
        var line = ""
        var digit = 0
        
        for _ in 1...leadingPadding {
            leadingPaddingString += " "
        }
        
        // Cyfry dziesiątek
        line = leadingPaddingString + " "
        for c in 1...cols {
            digit = c/10
            if digit == 0 {
                line += " "
            } else {
                line += "\(digit)"
            }
        }
        
        print(line)
        
        // Cyfry jedności
        line = leadingPaddingString + " "
        for c in 1...cols {
            digit = c%10
            line += "\(digit)"
        }
        
        print(line)
        
        for r in 0...rows+1 {
            line = ""

            if r == 0 || r == rows+1 {
                line += leadingPaddingString
            } else {
                line += String(r).leftPadding(toLength: leadingPadding)
            }
            
            for c in 0...cols+1 {
                switch board[r][c] {
                case .empty:
                    line += "."
                case .hit:
                    line += "!"
                case .ship:
                    line += "X"
                case .shot:
                    line += "*"
                case .none:
                    line += "?"
                case .notAllowed:
                    line += "+"
                case .rescue:
                    line += "O"
                }
            }
            
            print(line)
        }
    }
    
}


